level ="level"
time = "time"
time_range = "time_range"
year = "year"
month = "month"
day = "day"
dayofyear = "dayofyear"
hour = "hour"
ob_time = "ob_time"
ob_time_range = "ob_time_range"
ob_year = "ob_year"
ob_month = "ob_month"
ob_day = "ob_day"
ob_dayofyear = "ob_dayofyear"
ob_hour = "ob_hour"
dtime = "dtime"
dtime_range = "dtime_range"
dday = "dday"
dhour = "dhour"
id = "id"
lon = "lon"
lon_range = "lon_range"
lon_step = "lon_step"
lat = "lat"
lat_range = "lat_range"
lat_step = "lat_step"
last = "last"
last_range = "last_range"
last_step = "last_step"
grid = "grid"
value = "value"
drop_IV = "drop_IV"
gxy = "gxy"
gxyz = "gxyz"